<?php



/***
 * Name:       str2img
 * About:      displays a string of text as an image
 * Copyright:  (C) Monte Ohrt, All rights reserved.
 * Author:     Monte Ohrt, monte [at] ohrt [dot] com
 * License:    LGPL, see included license file  
 ***/

/**
 * str2img
 *
 * main object class
 *
 * @package		str2img
 * @author		Monte Ohrt
 */
 
class str2img {

 	/**
	 * $font
	 *
	 * the font filename
   * (must be in fonts folder)
	 *
	 * @access	public
	 */
  var $font = 'unifont.ttf';
  
 	/**
	 * $fontsize
	 *
	 * the font size in pixels
	 *
	 * @access	public
	 */
  var $fontsize = 10;

 	/**
	 * $fontangle
	 *
	 * the font angle, in degrees
	 *
	 * @access	public
	 */
  var $fontangle = 0;

 	/**
	 * $bgcolor
	 *
	 * the image background color in hex
	 *
	 * @access	public
	 */
  var $bgcolor = 'ffffff';
  
 	/**
	 * $textcolor
	 *
	 * the text color in hex
	 *
	 * @access	public
	 */
  var $textcolor = '000000';

 	/**
	 * $id
	 *
	 * the id value of the generated <img src> tag
	 *
	 * @access	public
	 */
  var $id = 'str2img';
  
  // internal vars

 	/**
	 * $_dir
	 *
	 * the system path to the installation directory.
   * This gets set in the contructor.
	 *
	 * @access	public
	 */
  var $_dir;

 	/**
	 * $_fontpath
	 *
	 * the system path to the font directory.
   * This gets set in the render() function.
	 *
	 * @access	public
	 */
  var $_fontpath;
  
 	/**
	 * $_mkimage_url
	 *
	 * the URL to the mkimage.php file.
   * This gets set in the contructor.
	 *
	 * @access	public
	 */  
  var $_mkimage_url;
  
 	/**
	 * $_def_*
	 *
	 * the default settings
	 *
	 * @access	public
	 */
  var $_def_font = 'unifont.ttf';
  var $_def_fontsize = 10;
  var $_def_fontangle = 0;
  var $_def_bgcolor = 'ffffff';
  var $_def_textcolor = '000000';
  
	/**
	 * str2img
	 *
	 * class constructor
	 *
	 * @access	public
	 */    
  function str2img() {
    // test that GD is installed with PNG support
    if(!function_exists('imagecreatetruecolor')||!function_exists('imagepng'))
      die('str2img requires the GD library with PNG support');
  
    $this->_dir = dirname(__FILE__) . DIRECTORY_SEPARATOR;
    $this->_mkimage_url = DIRECTORY_SEPARATOR . 'str2img' . DIRECTORY_SEPARATOR. 'mkimage.php';
  }
  
	/**
	 * imgsrc
	 *
	 * returns the <img src ...> tag for the image
	 *
	 * @access	public
	 * @param   string $text the text to be displayed
	 * @return  string the <img src ...> tag
	 */    
  function imgsrc($text) {  
    $text = $this->encode($text);
    $url = sprintf('%s?text=%s',$this->_mkimage_url,$text);
    if($this->bgcolor != $this->_def_bgcolor)
      $url .= sprintf('&bgcolor=%s',preg_replace('![^0-9a-fA-F]!','',$this->bgcolor));
    if($this->textcolor != $this->_def_textcolor)
      $url .= sprintf('&textcolor=%s',preg_replace('![^0-9a-fA-F]!','',$this->textcolor));
    if($this->font != $this->_def_font)
      $url .= sprintf('&font=%s',$this->font);
    if($this->fontangle != $this->_def_fontangle)
      $url .= sprintf('&fontangle=%s',$this->fontangle);
    if($this->fontsize != $this->_def_fontsize)
      $url .= sprintf('&fontsize=%s',$this->fontsize);
    return sprintf('<img src="%s" id="%s">',$url,$this->id);
  }
  
	/**
	 * render
	 *
	 * renders the image and sends to std output
	 *
	 * @access	public
	 * @param   string $text the text to be displayed
	 * @return  null
	 */    
  function render($text)
  {
    if(strlen($text)==0)
      return false;
      
    // sanitize font name, set the path
    $this->font = preg_replace('![^\w\.-]!','',$this->font);
    $this->_fontpath = $this->_dir . '..' . DIRECTORY_SEPARATOR . 'fonts' . DIRECTORY_SEPARATOR . $this->font;
    
    $text = $this->decode($text);
    
    // figure out bounding box for the text
    $bbox = $this->calculateTextBox($this->fontsize, $this->fontangle, $this->_fontpath, $text);
    
    // create image
    $im = imagecreatetruecolor($bbox['width'],$bbox['height']);
    
    // convert hex colors to rgb array
    $bgcolor = $this->hex2rgb($this->bgcolor);
    $textcolor = $this->hex2rgb($this->textcolor);
    
    // allocate image colors
    $bgcolor = imagecolorallocate($im, $bgcolor[0],$bgcolor[1],$bgcolor[2]);
    $textcolor = imagecolorallocate($im, $textcolor[0],$textcolor[1],$textcolor[2]);
    
    // Set the background color
    imagefilledrectangle($im, 0, 0, $bbox['width'], $bbox['height'], $bgcolor);
    
    // add text
    imagettftext($im,$this->fontsize,$this->fontangle,$bbox['left'],$bbox['top'],$textcolor,$this->_fontpath,$text);
    
    // output to browser
    header('Content-type: image/png');
    
    imagepng($im);
    imagedestroy($im);
  
    return true;
  }

	/**
	 * calculateTextBox
	 *
	 * calculate the bounding box for the text
	 *
   * @credit  Alexander Gavazov
   *
	 * @access	public
	 * @param   int $font_size in pixels
	 * @param   int $font_angle 0-360
	 * @param   string $font_file path to ttf font
	 * @param   string $text text to display on image
	 * @return  array bounding box data
	 */    

  function calculateTextBox($font_size, $font_angle, $font_file, $text) {
    $box = imagettfbbox($font_size, $font_angle, $font_file, $text);

    $min_x = min(array($box[0], $box[2], $box[4], $box[6]));
    $max_x = max(array($box[0], $box[2], $box[4], $box[6]));
    $min_y = min(array($box[1], $box[3], $box[5], $box[7]));
    $max_y = max(array($box[1], $box[3], $box[5], $box[7]));

    return array(
        'left' => ($min_x >= -1) ? -abs($min_x + 1) : abs($min_x + 2),
        'top' => abs($min_y),
        'width' => $max_x - $min_x,
        'height' => $max_y - $min_y,
        'box' => $box
    );
  }  
  
	/**
	 * decode
	 *
	 * decode a string
   *
	 * @access	public
	 * @param   string $string string to decode
	 * @return  string decoded text string
	 */    
  function decode($string) {    
    return unserialize($this->urlsafe_b64decode($string));
  }
  
	/**
	 * encode
	 *
	 * encodes a string
   *
	 * @access	public
	 * @param   string $string string to encode
	 * @return  string encoded text string
	 */    
  function encode($string) {
    return $this->urlsafe_b64encode(serialize($string));
  }

	/**
	 * urlsafe_b64encode
	 *
   * @credit  massimo dot scamarcia at gmail dot com
   *
	 * @access	public
	 * @param   string $string to encode
	 * @return  string encoded string
	 */    
  function urlsafe_b64encode($string) {
    return str_replace(array('+','/','='),array('-','_',''),base64_encode($string));
  }
  
	/**
	 * urlsafe_b64decode
	 *
   * @credit  massimo dot scamarcia at gmail dot com
   *
	 * @access	public
	 * @param   string $string to decode
	 * @return  string decoded string
	 */    
  function urlsafe_b64decode($string) {
      $mod4 = strlen($string) % 4;
      if ($mod4)
        $string .= substr('====', $mod4);
      return base64_decode(str_replace(array('-','_'),array('+','/'),$string));
  }
  
	/**
	 * hex2rgb
	 *
	 * converts hex value to an RGB array
	 *
   * @credit  anyexample.com
   *
	 * @access	public
	 * @param   string $color hex value of color
	 * @return  array (R,G,B) decimal value array
	 */    
  function hex2rgb($color) {
    if ($color[0] == '#')
      $color = substr($color, 1);
  
    if (strlen($color) == 6)
      list($r, $g, $b) = array($color[0].$color[1], $color[2].$color[3], $color[4].$color[5]);
    elseif (strlen($color) == 3)
      list($r, $g, $b) = array($color[0].$color[0], $color[1].$color[1], $color[2].$color[2]);
    else
      return false;
  
    $r = hexdec($r); $g = hexdec($g); $b = hexdec($b);
  
    return array($r, $g, $b);
  }

}
?>
